SystemOrganization addCategory: #'ICU-Core'!

!Date methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/29/2012 23:28'!
icuCalendar

	^ ICUCalendar new
		clear ;
		setDate: self year with: self monthIndex with: self dayOfMonth ;
		yourself! !

!Date methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/29/2012 22:57'!
icuFormatWith: aDateFormat

	^ aDateFormat formatSmalltalkDate: self! !

!ByteSymbol methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/30/2012 11:28'!
asUTF16Bytes

	^ ICUConverter byteStringAsUTF16: self! !

!WideSymbol methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/30/2012 11:29'!
asUTF16Bytes

	^ ICUConverter wideStringAsUTF16: self! !

!ByteArray methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 14:37'!
asTwoByteString

	^ TwoByteString fromByteArray: self! !

SharedPool subclass: #ICUConstants
	instanceVariableNames: ''
	classVariableNames: 'UCAL_AM_PM UCAL_DATE UCAL_DAY_OF_WEEK UCAL_DAY_OF_WEEK_IN_MONTH UCAL_DAY_OF_YEAR UCAL_DST_OFFSET UCAL_ERA UCAL_HOUR UCAL_HOUR_OF_DAY UCAL_MILLISECOND UCAL_MINUTE UCAL_MONTH UCAL_SECOND UCAL_WEEK_OF_MONTH UCAL_WEEK_OF_YEAR UCAL_YEAR UCAL_ZONE_OFFSET UDAT_DEFAULT UDAT_FULL UDAT_FULL_RELATIVE UDAT_IGNORE UDAT_LONG UDAT_LONG_RELATIVE UDAT_MEDIUM UDAT_MEDIUM_RELATIVE UDAT_NONE UDAT_RELATIVE UDAT_SHORT UDAT_SHORT_RELATIVE'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUConstants commentStamp: '<historical>' prior: 0!
An ICUConstants is a pool dictionary with ICU related constants.!

!ICUConstants class methodsFor: 'initialization' stamp: 'JanVanDeSandt 8/29/2012 11:15'!
initialize

	self initializeCalendarConstants.
	self initializeDateFormatConstants.! !

!ICUConstants class methodsFor: 'initialization' stamp: 'JanVanDeSandt 8/29/2012 09:48'!
initializeCalendarConstants

	UCAL_ERA := 0.
	UCAL_YEAR := 1.
	UCAL_MONTH := 2.
	UCAL_WEEK_OF_YEAR := 3.
	UCAL_WEEK_OF_MONTH := 4.
	UCAL_DATE := 5.
	UCAL_DAY_OF_YEAR := 6.
	UCAL_DAY_OF_WEEK := 7.
	UCAL_DAY_OF_WEEK_IN_MONTH := 8.
	UCAL_AM_PM := 9.
	UCAL_HOUR := 10.
	UCAL_HOUR_OF_DAY := 11.
	UCAL_MINUTE := 12.
	UCAL_SECOND := 13.
	UCAL_MILLISECOND := 14.
	UCAL_ZONE_OFFSET := 15.
	UCAL_DST_OFFSET := 16
	
! !

!ICUConstants class methodsFor: 'initialization' stamp: 'JanVanDeSandt 8/29/2012 11:17'!
initializeDateFormatConstants

	UDAT_FULL := 0.
	UDAT_LONG := 1.
	UDAT_MEDIUM := 2.
	UDAT_SHORT := 3.
	UDAT_DEFAULT := UDAT_MEDIUM.
	UDAT_RELATIVE := 1 << 7.	
	UDAT_FULL_RELATIVE := UDAT_FULL bitOr: UDAT_RELATIVE.
	UDAT_LONG_RELATIVE := UDAT_LONG bitOr: UDAT_RELATIVE.
	UDAT_MEDIUM_RELATIVE := UDAT_MEDIUM bitOr: UDAT_RELATIVE.
	UDAT_SHORT_RELATIVE := UDAT_SHORT bitOr: UDAT_RELATIVE.
	UDAT_NONE := -1.
	UDAT_IGNORE := -2. 
! !

!ByteString class methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 16:54'!
icuConverter
	"Answer an ICU Converter that can be used to convert instances of this class to and from Unicode or other code pages."
	
	^ ICUConverter defaultByteStringConverter! !

!ByteString methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 14:39'!
asUTF16Bytes

	^ ICUConverter byteStringAsUTF16: self! !

!WideString class methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 15:45'!
fromLittleEndianByteArray: aByteArray size: aSize
	"Answer a WideString of size <aSize> created from a LE enocoded ByteArray."

	| inst |
	aSize \\ 4 = 0 ifFalse: [^ self error: 'Invalid size' ].
	inst := self new: aSize // 4.
	4 to: aSize by: 4 do: [:i |
		inst basicAt: i // 4
			put: (aByteArray at: i - 3) + 
				((aByteArray at: i - 2) << 8) +
				 ((aByteArray at: i - 1) << 16) +
				((aByteArray at: i) << 24)
	].
	^ inst
! !

!WideString class methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 16:54'!
icuConverter
	"Answer an ICU Converter that can be used to convert instances of this class to and from Unicode or other code pages."
	
	^ ICUConverter defaultWideStringConverter! !

!WideString methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 14:40'!
asUTF16Bytes

	^ ICUConverter wideStringAsUTF16: self! !

Object subclass: #ICUInterface
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUInterface commentStamp: '<historical>' prior: 0!
An abstract class that provides access to the ICU native classes. Concrete subclasses provide the actual implementation. One of these subclasses can be the Current.!

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:16'!
clearCurrent
	"Reset the current variable."
	
	Current := nil
	! !

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:07'!
current
	"Answer a concrete subclass of the receiver"

	^ Current! !

!ICUInterface class methodsFor: 'testing' stamp: 'JanVanDeSandt 3/2/2012 14:21'!
isCurrent

	^ Current = self! !

!ICUInterface class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:16'!
makeCurrent
	"Make the receiver the current interface class."
	
	Current := self
	! !

!ICUInterface class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:01'!
nativeCalendarClass

	self subclassResponsibility ! !

!ICUInterface class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:01'!
nativeConverterClass

	self subclassResponsibility ! !

!ICUInterface class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/9/2012 11:06'!
nativeLocaleClass

	self subclassResponsibility ! !

Object subclass: #ICUModules
	instanceVariableNames: 'platformName commonModule i18nModule versionSuffix'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUModules commentStamp: 'JanVanDeSandt 2/25/2012 15:32' prior: 0!
ICUModules contains the ICU4C library/module names and a possible versionSuffix.

The ICU - International Components for Unicode Library (icu4c) can be downloaded from: http://site.icu-project.org/

Win32
	Add the ICU bin directory to the PATH environment variable.
	Note that the ICU libraries on Windows require the Microsoft Visual C++ 2010 Redistributable Package

MaxOSX
	Get the 32bit versions of the ICU libraries
	For example by using MacPorts (on Snow Leapard set the build_arch configuration property to i386 to get 32bit libraries).
	Create a symbolic link to the libraries from inside the Resources directory of the VM. For example
		ln -s /opt/local/lib/libicuuc.dylib .
		ln -s /opt/local/lib/libicui18n .
	
Unix
	On Ubuntu 10.10 you can install the library:
		sudo apt-get install libicu42!

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:48'!
clear

	Default := nil! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:55'!
default

	^ Default ifNil: [ Default := self determineDefault ]! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:48'!
default: anObject

	Default := anObject! !

!ICUModules class methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:55'!
determineDefault

	Smalltalk os isWin32 
		ifTrue: [ ^ self winVersion48 ].
		
	Smalltalk os isMacOSX
		ifTrue: [ ^ self osxVersion48 ].
		
	^ self unixVersion42! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 2/25/2012 16:06'!
initialize 
	
	Smalltalk addToStartUpList: self! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:08'!
osxVersion46
	"self default: self osxVersion46"

	^self new setCommonModule: 'libicuuc.dylib' i18nModule: 'libicui18n.dylib' versionSuffix: '_46'! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
osxVersion48
	"self default: self osxVersion48"

	^self new setCommonModule: 'libicuuc.dylib' i18nModule: 'libicui18n.dylib' versionSuffix: '_48'! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 2/25/2012 16:00'!
startUp: resuming
	"Clear the default instance if it was created for another platform."

	resuming
		ifTrue: [
			(Default notNil and: [ Default platformName ~= Smalltalk os platformName ])
				ifTrue: [ Default := nil ] ]! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
unixVersion42
	"self default: self unixVersion42"

	^self new setCommonModule: 'libicuuc.so.42' i18nModule: 'libicui18n.so.42' versionSuffix: '_4_2'! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/3/2012 23:55'!
unixVersion44
	"self default: self unixVersion44"

	^self new setCommonModule: 'libicuuc.so.44' i18nModule: 'libicui18n.so.44' versionSuffix: '_44'! !

!ICUModules class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 3/2/2012 11:08'!
unload 

	self clear.
	Smalltalk removeFromStartUpList: self! !

!ICUModules class methodsFor: 'resources' stamp: 'JanVanDeSandt 2/25/2012 16:09'!
winVersion48
	"self default: self winVersion48"

	^self new setCommonModule: 'icuuc48.dll' i18nModule: 'icuin48.dll' versionSuffix: '_48'! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
commonModule

	^commonModule ! !

!ICUModules methodsFor: 'testing' stamp: 'JanVanDeSandt 3/7/2012 08:22'!
hasVersionSuffix

	^versionSuffix notNil and: [ versionSuffix notEmpty ]! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
i18nModule

	^ i18nModule ! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:51'!
platformName

	^ platformName ! !

!ICUModules methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 2/25/2012 15:50'!
setCommonModule: aCommonModuleName i18nModule: anI18NModuleName versionSuffix: aVersionSuffix

	platformName := Smalltalk os platformName.
	commonModule := aCommonModuleName.
	i18nModule := anI18NModuleName.
	versionSuffix := aVersionSuffix ! !

!ICUModules methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/25/2012 15:52'!
versionSuffix

	^ versionSuffix! !

Object subclass: #ICUObject
	instanceVariableNames: 'native'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

ICUObject subclass: #ICUCalendar
	instanceVariableNames: 'locale timeZone'
	classVariableNames: ''
	poolDictionaries: 'ICUConstants'
	category: 'ICU-Core'!

!ICUCalendar class methodsFor: 'time zones' stamp: 'JanVanDeSandt 3/2/2012 14:34'!
allTimeZones

	^ self nativeClass allTimeZones ! !

!ICUCalendar class methodsFor: 'time zones' stamp: 'JanVanDeSandt 3/2/2012 14:33'!
defaultTimeZone

	^ self nativeClass getDefaultTimeZone! !

!ICUCalendar class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:09'!
nativeClass

	^ self icuInterface nativeCalendarClass ! !

!ICUCalendar class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 3/9/2012 11:19'!
new

	^ self newWithLocale: ICULocale defaultLocaleName timeZone: self defaultTimeZone ! !

!ICUCalendar class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 8/27/2012 23:57'!
newWithLocale: aLocaleString timeZone: aTimeZoneString

	^ self basicNew initializeWithLocale: aLocaleString timeZone: aTimeZoneString! !

!ICUCalendar methodsFor: 'actions' stamp: 'JanVanDeSandt 8/29/2012 23:06'!
clear

	self native clear! !

!ICUCalendar methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/28/2012 00:09'!
createNative

	native := self nativeClass openDefaultWithLocale: locale timezone: timeZone.
	native initializeLifecycle! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:42'!
dayOfMonth

	^self get: UCAL_DATE ! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:42'!
dayOfYear

	^self get: UCAL_DAY_OF_YEAR! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:42'!
dstOffset
	"Field number indicating the daylight savings offset in milliseconds. "
	
	^self get: UCAL_DST_OFFSET ! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/27/2012 23:59'!
get: ucoConstant

	^ self native get: ucoConstant! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:49'!
hour	
	
	^self get: UCAL_HOUR! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:49'!
hourOfDay	
	
	^self get: UCAL_HOUR_OF_DAY! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 10:06'!
hourOfDay: anInteger
	
	self set: UCAL_HOUR_OF_DAY to: anInteger! !

!ICUCalendar methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/27/2012 23:46'!
initializeWithLocale: aLocaleString timeZone: aTimeZoneString

	locale := aLocaleString.
	timeZone := aTimeZoneString.
	self createNative ! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:50'!
milliSecond	
	
	^self get: UCAL_MILLISECOND! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 10:09'!
milliSecond: anInteger
	
	self set: UCAL_MILLISECOND to: anInteger! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 15:00'!
millis
	^ self native millis asInteger! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 00:19'!
millis: anInteger 
	self native millis: anInteger! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:49'!
minute	
	
	^self get: UCAL_MINUTE ! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:49'!
second	
	
	^self get: UCAL_SECOND! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 10:07'!
set: ucoConstant to: anInteger

	self native set: ucoConstant to: anInteger! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 23:15'!
setDate: aYear with: aMonth with: aDayOfMonth

	self native setDate: aYear with: aMonth with: aDayOfMonth! !

!ICUCalendar methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 00:29'!
setDateTime: aYear with: aMonth with: aDayOfMonth with: anHour with: aMinute with: aSecond

	self native setDateTime: aYear with: aMonth with: aDayOfMonth with: anHour with: aMinute with: aSecond! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:39'!
weekOfMonth

	^self get: UCAL_WEEK_OF_MONTH ! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:39'!
weekOfYear

	^self get: UCAL_WEEK_OF_YEAR! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/29/2012 09:39'!
year

	^self get: UCAL_YEAR! !

!ICUCalendar methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 14:43'!
zoneOffset
	"Field number indicating the daylight savings offset in milliseconds. "
	
	^self get: UCAL_ZONE_OFFSET ! !

ICUObject subclass: #ICUConverter
	instanceVariableNames: 'name'
	classVariableNames: 'Latin1Converter UTF32LEConverter UTF8Converter'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUConverter commentStamp: '<historical>' prior: 0!
An ICUConverter is a converter ...!

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:04'!
allNames

	^ self nativeClass allNames! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 14:08'!
byteStringAsUTF16: aByteString
	"Answer a ByteArray with a UTF16 encoded version of <aByteString>."

	^ self latin1Converter toUTF16: aByteString
	
	"self byteStringAsUTF16: 'Hello'"! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 14:32'!
byteStringAsUTF8: aByteString
	"Answer a ByteArray with a UTF8 encoded version of <aByteString>."

	^ self utf8Converter fromType: self defaultByteStringConverterType source: aByteString
	
	"self byteStringAsUTF8: 'Hello'"
	
! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 15:14'!
byteStringFromUTF16: aSource

	| buffer length |
	
	buffer := ByteArray new: (aSource byteSize / 2).
	length := self latin1Converter fromUTF16: aSource destination: buffer.
	^(ByteString new: length) replaceFrom: 1 to: length with: buffer startingAt: 1! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/4/2012 15:15'!
byteStringFromUTF8: aSource

	| buffer length |
	
	buffer := ByteArray new: aSource byteSize.

	length := self utf8Converter toType: self defaultByteStringConverterType source: aSource destination: buffer.
	^(ByteString new: length) replaceFrom: 1 to: length with: buffer startingAt: 1! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 3/9/2012 16:30'!
clear
	"self clear"

	UTF8Converter := nil.
	UTF32LEConverter := nil.
	Latin1Converter := nil.! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeLatin1

	^ 3! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF16BE

	^ 5! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF16LE

	^ 6! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:32'!
converterTypeUTF32

	^ 30! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF32BE

	^ 7! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:10'!
converterTypeUTF32LE

	^ 8! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:09'!
converterTypeUTF8

	^ 4! !

!ICUConverter class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/9/2012 16:43'!
defaultByteStringConverter

	^ self latin1Converter! !

!ICUConverter class methodsFor: 'resources' stamp: 'jvds 2/26/2012 15:09'!
defaultByteStringConverterType

	^ self converterTypeLatin1! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/4/2012 14:15'!
defaultName

	^ self nativeClass getDefaultName! !

!ICUConverter class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/9/2012 16:43'!
defaultWideStringConverter

	^ self utf32leConverter ! !

!ICUConverter class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/9/2012 14:19'!
defaultWideStringConverterType

	^ self converterTypeUTF32LE ! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'jvds 2/26/2012 20:53'!
initialize

	Smalltalk addToShutDownList: self! !

!ICUConverter class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/9/2012 14:01'!
latin1Converter

	^ Latin1Converter ifNil: [ Latin1Converter := self openLatin1 ]! !

!ICUConverter class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/2/2012 14:09'!
nativeClass

	^ self icuInterface nativeConverterClass ! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/28/2012 00:03'!
open: aConverterName

	^ self new initializeWithName: aConverterName! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/9/2012 16:46'!
openBOCU1
	"BOCU-1: Binary Ordered Compression for Unicode"

	^ self open: 'BOCU-1'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:03'!
openISO_8859_1

	^ self open: 'ISO-8859-1'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/4/2012 14:47'!
openLatin1

	^ self openISO_8859_1! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/9/2012 14:15'!
openUTF32

	^ self open: 'UTF-32'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/9/2012 14:15'!
openUTF32LE

	^ self open: 'UTF-32LE'! !

!ICUConverter class methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 14:04'!
openUTF8

	^ self open: 'UTF-8'! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'jvds 2/26/2012 20:54'!
shutDown: quitting

	self clear! !

!ICUConverter class methodsFor: 'class initialization' stamp: 'JanVanDeSandt 3/2/2012 15:26'!
unload

	Smalltalk removeFromShutDownList: self! !

!ICUConverter class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/9/2012 14:15'!
utf32leConverter

	^ UTF32LEConverter ifNil: [ UTF32LEConverter := self openUTF32LE ]! !

!ICUConverter class methodsFor: 'accessing' stamp: 'jvds 2/26/2012 15:08'!
utf8Converter

	^ UTF8Converter ifNil: [ UTF8Converter := self openUTF8 ]! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 14:17'!
wideStringAsUTF16: aWideString
	"Answer a ByteArray with a UTF16 encoded version of <aWideString>."

	^ self utf32leConverter toUTF16: aWideString
	
	"self wideStringAsUTF16: 'Hello' asWideString"! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 14:33'!
wideStringAsUTF8: aWideString
	"Answer a ByteArray with a UTF8 encoded version of <aWideString>."

	^ self utf8Converter fromType: self defaultWideStringConverterType source: aWideString
	
	"self wideStringAsUTF8: 'Hello' asWideString"
	
! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 15:43'!
wideStringFromUTF16: aSource
	"Answer a WideString."
	
	| buffer length |
	
	buffer := ByteArray new: aSource byteSize * 2.
	length := self utf32leConverter fromUTF16: aSource destination: buffer. 
	^ WideString fromLittleEndianByteArray: buffer size: length 

	! !

!ICUConverter class methodsFor: 'converting' stamp: 'JanVanDeSandt 3/9/2012 15:43'!
wideStringFromUTF8: aSource
	"Answer a WideString."
	
	| buffer length |
	
	buffer := ByteArray new: aSource byteSize * 4.
	length := self utf8Converter toType: self defaultWideStringConverterType source: aSource destination: buffer.
	^ WideString fromLittleEndianByteArray: buffer size: length 

	! !

!ICUConverter methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/28/2012 00:09'!
createNative

	native := self nativeClass open: name.
	native initializeLifecycle! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 14:33'!
fromType: aConverterType source: aSource

	| buffer length |

	buffer := ByteArray new: aSource size + 10.
	length := native fromAlgorithmic: aConverterType source: aSource destination: buffer.
	^ buffer copyFrom: 1 to: length.! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/4/2012 14:30'!
fromType: aConverterType source: aSource destination: aBuffer

	^ native fromAlgorithmic: aConverterType source: aSource destination: aBuffer! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 14:44'!
fromUTF16: aSource
	"Convert the Unicode string <aSource> into a codepage string."

	| buffer length |

	buffer := ByteArray new: aSource size + 10.
	length := self fromUTF16: aSource destination: buffer.
	^ buffer copyFrom: 1 to: length! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 14:28'!
fromUTF16: aSource destination: aBuffer
	"Convert the Unicode data in <aSource> into a codepage string."

	^ native fromUChars: aSource destination: aBuffer! !

!ICUConverter methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/28/2012 00:02'!
initializeWithName: aString

	name := aString.
	self createNative! !

!ICUConverter methodsFor: 'testing' stamp: 'jvds 2/26/2012 14:08'!
isAmbiguous

	^ native isAmbiguous ! !

!ICUConverter methodsFor: 'testing' stamp: 'JanVanDeSandt 3/4/2012 14:16'!
isFixedWidth
	"Available in ICU version 4.8 or higher."

	^ native isFixedWidth! !

!ICUConverter methodsFor: 'accessing' stamp: 'jvds 2/26/2012 15:28'!
name

	^ native name! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 16:27'!
substitutionCharacters

	| buffer length |
	
	buffer := ByteArray new: 20.
	length := native getSubstChars: buffer.
	^buffer copyFrom: 1 to: length! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/4/2012 14:45'!
toType: aConverterType source: aSource destination: aBuffer

	^ native toAlgorithmic: aConverterType source: aSource destination: aBuffer! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 14:31'!
toUTF16: aString
	"Convert the codepage string <aString> into a Unicode string. "

	| buffer length |

	buffer := ByteArray new: aString size * 2.
	length := self toUTF16: aString destination: buffer.
	^length = (buffer size / 2)
		ifTrue: [ buffer ]
		ifFalse: [ buffer copyFrom: 1 to: length * 2 ]! !

!ICUConverter methodsFor: 'actions' stamp: 'JanVanDeSandt 3/9/2012 14:30'!
toUTF16: aString destination: aBuffer
	"Convert the codepage string <aString> into a Unicode string. "

	^ native toUChars: aString destination: aBuffer! !

!ICUConverter methodsFor: 'accessing' stamp: 'jvds 2/26/2012 15:29'!
type

	^ native type! !

ICUObject subclass: #ICUFormat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

ICUFormat subclass: #ICUDateFormat
	instanceVariableNames: 'timeStyle dateStyle locale timeZoneId pattern'
	classVariableNames: ''
	poolDictionaries: 'ICUConstants'
	category: 'ICU-Core'!

!ICUDateFormat class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 8/29/2012 14:27'!
fullDate

	^ super new initializeWith: UDAT_NONE with: UDAT_FULL with: nil with: '' with: ''! !

!ICUDateFormat class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 8/29/2012 14:26'!
fullTime

	^ super new initializeWith: UDAT_FULL with: UDAT_NONE with: nil with: '' with: ''! !

!ICUDateFormat class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 8/29/2012 14:24'!
fullTimeFullDate

	^ super new initializeWith: UDAT_FULL with: UDAT_FULL with: nil with: '' with: ''! !

!ICUDateFormat class methodsFor: 'resources' stamp: 'JanVanDeSandt 8/29/2012 11:21'!
nativeClass

	^ self icuInterface nativeDateFormatClass ! !

!ICUDateFormat class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 8/29/2012 13:45'!
new

	^ super new initializeWith: UDAT_DEFAULT with: UDAT_DEFAULT with: nil with: '' with: ''! !

!ICUDateFormat methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/29/2012 12:12'!
createNative

	native := self nativeClass open: timeStyle with: dateStyle with: locale with: timeZoneId with: pattern.
	native initializeLifecycle! !

!ICUDateFormat methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 15:02'!
format: anInteger

	^ self native format: anInteger asFloat! !

!ICUDateFormat methodsFor: 'actions' stamp: 'JanVanDeSandt 8/29/2012 23:05'!
formatSmalltalkDate: aDate! !

!ICUDateFormat methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/29/2012 12:09'!
initializeWith: aTimeStyle with: aDateStyle with: aLocale with: aTimeZoneId with: aPattern

	timeStyle := aTimeStyle.
	dateStyle := aDateStyle.
	locale := aLocale.
	timeZoneId := aTimeZoneId.
	pattern := aPattern.
	self createNative ! !

!ICUDateFormat methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 22:26'!
isLenient

	^ self native isLenient! !

!ICUDateFormat methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 22:26'!
lenient: aBoolean

	self native lenient: aBoolean! !

!ICUDateFormat methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 15:00'!
parse: aString

	^ (self native parse: aString) asInteger! !

ICUObject subclass: #ICULocale
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/25/2012 21:29'!
allAvailable

	^ self nativeClass getAllAvailable ! !

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/9/2012 11:19'!
defaultLocaleName

	^ self nativeClass getDefault! !

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/25/2012 21:20'!
defaultLocaleName: aString

	self nativeClass setDefault: aString! !

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/25/2012 21:27'!
displayName: aLocale in: inLocale

	^ self nativeClass getDisplayName: aLocale in: inLocale! !

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/25/2012 21:29'!
isoCountries

	^ self nativeClass isoCountries ! !

!ICULocale class methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/25/2012 21:30'!
isoLanguages

	^ self nativeClass isoLanguages! !

!ICULocale class methodsFor: 'resources' stamp: 'JanVanDeSandt 3/9/2012 11:06'!
nativeClass

	^ self icuInterface nativeLocaleClass ! !

!ICUObject class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 3/2/2012 14:06'!
icuInterface

	^ ICUInterface current! !

!ICUObject methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/28/2012 10:06'!
close
	"Optional - Close and clear the native object. After this operation this object is no longer usable.
	Closing is not required. Foreign resources will otherwise be released during finelization."

	native notNil
		ifTrue: [ native close. native := nil ]! !

!ICUObject methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 8/27/2012 23:45'!
createNative

	self subclassResponsibility ! !

!ICUObject methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/27/2012 23:45'!
native

	native isValid
		ifFalse: [ self createNative ].
	^ native! !

!ICUObject methodsFor: 'resources' stamp: 'JanVanDeSandt 8/27/2012 23:47'!
nativeClass

	^ self class nativeClass ! !

Error subclass: #ICUError
	instanceVariableNames: ''
	classVariableNames: 'ErrorTextDictionary'
	poolDictionaries: ''
	category: 'ICU-Core'!

ICUError subclass: #ICUBufferOverflowError
	instanceVariableNames: 'length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUError methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 15:47'!
clear

	ErrorTextDictionary := nil! !

!ICUError methodsFor: 'as yet unclassified' stamp: 'jvds 2/26/2012 15:46'!
errorTextDictionary

	^ ErrorTextDictionary  ifNil: [
		ErrorTextDictionary := Dictionary new
			at: 1 put: 'U_ILLEGAL_ARGUMENT_ERROR' ;
			at: 3 put: 'U_INVALID_FORMAT_ERROR' ;
			at: 15 put: 'U_BUFFER_OVERFLOW_ERROR' ;
			yourself ]! !

!String class methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 16:52'!
icuConverter
	"Answer an ICU Converter that can be used to convert instances of this class to and from Unicode or other code pages."
	
	self subclassResponsibility ! !

!String class methodsFor: '*ICU-Core' stamp: 'JanVanDeSandt 3/9/2012 16:57'!
icuFromUTF8: aSource! !

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/30/2012 11:20'!
asTwoByteString 

	^ TwoByteString fromByteArray: self asUTF16Bytes! !

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 8/30/2012 11:28'!
asUTF16Bytes
	"Answer a ByteArray with the UTF-16 encoded data for this String."

	self subclassResponsibility ! !

!String methodsFor: '*icu-core' stamp: 'JanVanDeSandt 2/21/2012 22:18'!
isTwoByteString

	^ false! !

String subclass: #TwoByteString
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!TwoByteString class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 2/21/2012 22:02'!
fromByteArray: aByteArray 

	aByteArray size \\ 2 = 0 ifFalse: [^ ByteString fromByteArray: aByteArray ].
	^ self basicNew setBytes: aByteArray! !

!TwoByteString class methodsFor: 'instance creation' stamp: 'JanVanDeSandt 2/21/2012 22:10'!
fromByteArray: aByteArray charSize: anInteger

	aByteArray size = (anInteger*2)
		ifTrue: [ ^ self fromByteArray: aByteArray ].
		
	^self basicNew setBytes: (aByteArray copyFrom: 1 to: anInteger*2)! !

!TwoByteString class methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 2/21/2012 21:56'!
new: aSize

	^ self basicNew setBytes: (ByteArray new: aSize * 2)! !

!TwoByteString methodsFor: 'as yet unclassified' stamp: 'JanVanDeSandt 8/29/2012 14:57'!
asTwoByteString

	^ self! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 8/30/2012 11:34'!
asUTF16Bytes
	"Answer a ByteArray with the UTF-16 encoded data for this String."

	^ bytes! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:52'!
at: index

	^ Character value: (bytes at: index*2 - 1) + ((bytes at: index*2) << 8)! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:59'!
at: index put: aCharacter

	bytes at: (index*2 - 1) put: aCharacter asInteger \\ 256.
	bytes at: (index*2) put: aCharacter asInteger >> 8.
	
	^ aCharacter! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 3/2/2012 15:03'!
basicAt: index

	^ (bytes at: index*2 - 1) + ((bytes at: index*2) << 8)! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 22:44'!
bytes

	^ bytes! !

!TwoByteString methodsFor: 'testing' stamp: 'JanVanDeSandt 2/21/2012 22:19'!
isTwoByteString

	^ true! !

!TwoByteString methodsFor: 'initialize-release' stamp: 'JanVanDeSandt 2/21/2012 21:46'!
setBytes: aByteArray

	bytes := aByteArray! !

!TwoByteString methodsFor: 'accessing' stamp: 'JanVanDeSandt 2/21/2012 21:47'!
size

	^ bytes size / 2! !
ICUConstants initialize!
ICUConverter initialize!
ICUModules initialize!
